=== modified file 'lib/canonical/launchpad/components/decoratedresultset.py'
--- lib/canonical/launchpad/components/decoratedresultset.py	2011-07-21 15:59:49 +0000
+++ lib/canonical/launchpad/components/decoratedresultset.py	2011-07-28 11:13:27 +0000
@@ -11,8 +11,10 @@
 from lazr.delegates import delegates
 from storm import Undef
 from storm.zope.interfaces import IResultSet
-from zope.interface import implements
-from zope.security.proxy import removeSecurityProxy
+from zope.security.proxy import (
+    ProxyFactory,
+    removeSecurityProxy,
+    )
 
 
 class DecoratedResultSet(object):
@@ -175,3 +177,18 @@
     def getPlainResultSet(self):
         """Return the plain Storm result set."""
         return self.result_set
+
+    def find(self, *args, **kwargs):
+        """See `IResultSet`.
+
+        :return: The decorated version of the returned result set.
+        """
+        naked_result_set = removeSecurityProxy(self.result_set)
+        if naked_result_set is not self.result_set:
+            naked_new_result_set = naked_result_set.find(*args, **kwargs)
+            new_result_set = ProxyFactory(naked_new_result_set)
+        else:
+            new_result_set = self.result_set.find(*args, **kwargs)
+        return DecoratedResultSet(
+            new_result_set, self.result_decorator, self.pre_iter_hook,
+            self.slice_info)

=== modified file 'lib/canonical/launchpad/components/tests/decoratedresultset.txt'
--- lib/canonical/launchpad/components/tests/decoratedresultset.txt	2010-10-18 22:24:59 +0000
+++ lib/canonical/launchpad/components/tests/decoratedresultset.txt	2011-07-28 11:13:27 +0000
@@ -136,3 +136,19 @@
     >>> isinstance(decorated_result_set[0:3], DecoratedResultSet)
     True
 
+== find() ==
+
+DecoratedResultSet.find() returns another DecoratedResultSet containing
+a refined query.
+
+    >>> result_set = store.find(Distribution)
+    >>> proxied_result_set = ProxyFactory(result_set)
+    >>> decorated_result_set = DecoratedResultSet(
+    ...     proxied_result_set, result_decorator)
+    >>> ubuntu_distros = removeSecurityProxy(decorated_result_set).find(
+    ...     "Distribution.name like 'ubuntu%'")
+    >>> for dist in ubuntu_distros:
+    ...     dist
+    u'Dist name is: ubuntu'
+    u'Dist name is: ubuntutest'
+

=== modified file 'lib/canonical/launchpad/webapp/batching.py'
--- lib/canonical/launchpad/webapp/batching.py	2011-07-27 16:21:22 +0000
+++ lib/canonical/launchpad/webapp/batching.py	2011-07-28 11:13:27 +0000
@@ -231,8 +231,9 @@
 
     def getEndpointMemos(self, batch):
         """See `IRangeFactory`."""
-        lower = self.getOrderValuesFor(batch.first())
-        upper = self.getOrderValuesFor(batch.last())
+        lower = self.getOrderValuesFor(self.plain_resultset[0])
+        upper = self.getOrderValuesFor(
+            self.plain_resultset[batch.trueSize - 1])
         return (
             simplejson.dumps(lower, cls=DateTimeJSONEncoder),
             simplejson.dumps(upper, cls=DateTimeJSONEncoder),

=== modified file 'lib/canonical/launchpad/webapp/tests/test_batching.py'
--- lib/canonical/launchpad/webapp/tests/test_batching.py	2011-07-27 16:21:22 +0000
+++ lib/canonical/launchpad/webapp/tests/test_batching.py	2011-07-28 11:13:27 +0000
@@ -196,10 +196,26 @@
         range_factory = StormRangeFactory(resultset)
         first, last = range_factory.getEndpointMemos(batchnav.batch)
         expected_first = simplejson.dumps(
-            range_factory.getOrderValuesFor(batchnav.batch.first()),
-            cls=DateTimeJSONEncoder)
-        expected_last = simplejson.dumps(
-            range_factory.getOrderValuesFor(batchnav.batch.last()),
+            [resultset[0].name], cls=DateTimeJSONEncoder)
+        expected_last = simplejson.dumps(
+            [resultset[2].name], cls=DateTimeJSONEncoder)
+        self.assertEqual(expected_first, first)
+        self.assertEqual(expected_last, last)
+
+    def test_getEndpointMemos__decorated_result_set(self):
+        # getEndpointMemos() works for DecoratedResultSet
+        # instances too.
+        resultset = self.makeDecoratedStormResultSet()
+        resultset.order_by(LibraryFileAlias.id)
+        request = LaunchpadTestRequest()
+        batchnav = BatchNavigator(
+            resultset, request, size=3, range_factory=StormRangeFactory)
+        range_factory = StormRangeFactory(resultset)
+        first, last = range_factory.getEndpointMemos(batchnav.batch)
+        expected_first = simplejson.dumps(
+            [resultset.getPlainResultSet()[0][1].id], cls=DateTimeJSONEncoder)
+        expected_last = simplejson.dumps(
+            [resultset.getPlainResultSet()[2][1].id],
             cls=DateTimeJSONEncoder)
         self.assertEqual(expected_first, first)
         self.assertEqual(expected_last, last)
@@ -400,6 +416,15 @@
         sliced_result = range_factory.getSlice(3, memo, forwards=False)
         self.assertEqual(expected, list(sliced_result))
 
+    def test_getSlice__decorated_resultset(self):
+        resultset = self.makeDecoratedStormResultSet()
+        resultset.order_by(LibraryFileAlias.id)
+        all_results = list(resultset)
+        memo = simplejson.dumps([resultset.getPlainResultSet()[0][1].id])
+        range_factory = StormRangeFactory(resultset)
+        sliced_result = range_factory.getSlice(3, memo)
+        self.assertEqual(all_results[1:4], list(sliced_result))
+
 
 def test_suite():
     return TestLoader().loadTestsFromName(__name__)

